/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.panels;

import java.awt.EventQueue;
import java.awt.Rectangle;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import javax.swing.JComponent;
import javax.swing.SwingUtilities;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.LayerInfo;
import nl.cloudfarming.client.geoviewer.LayerObject;
import nl.cloudfarming.client.geoviewer.layers.LayerObjectNode;
import nl.cloudfarming.client.geoviewer.layers.LayerNode;
import org.jdesktop.swingx.JXMapViewer;
import org.netbeans.api.visual.model.ObjectScene;
import org.netbeans.api.visual.widget.LayerWidget;
import org.netbeans.api.visual.widget.Widget;
import org.openide.nodes.Node;

/**
 * The WidgetsPanel is used for holding the dynamic map objects, so the user
 * can interact with them, for example showing a tooltip when hover over the
 * object or using a right mouse click context menu (to for example identify
 * the object).
 * 
 * @author Timon Veenstra
 */
public class LayerPanel extends LayerInfo {

    private final ObjectScene scene = new ObjectScene();
    private final JComponent view = scene.createView();
    private final LayerWidget mainLayer = new LayerWidget(scene);
    private MouseEventForwarder mouseEventForwarder;
    private final JXMapViewer mapViewer;
    private final RootMapPanelController controller;
    private final Layer layer;

    public LayerPanel(final RootMapPanelController controller, final JComponent forwardEventsTo, Layer layer) {
        this.layer = layer;
        this.mapViewer = controller.getViewer();
        this.controller = controller;
        this.scene.setOpaque(false);
        this.scene.addChild(this.mainLayer);
        this.scene.setMaximumBounds(new Rectangle(0, 0, Integer.MAX_VALUE, Integer.MAX_VALUE));

        if (forwardEventsTo != null) {
            this.mouseEventForwarder = new MouseEventForwarder(this.scene, forwardEventsTo);
            this.view.addMouseListener(this.mouseEventForwarder);
            this.view.addMouseMotionListener(this.mouseEventForwarder);
            this.view.addMouseWheelListener(this.mouseEventForwarder);
        }

        addObjects(layer.getObjects());
//        addNodes(layerNode.getChildren().getNodes());
    }

    public JComponent getView() {
        return this.view;
    }

    /**
     * remove objects
     * if remove is called from the event dispatch thread it ill be
     * executed immediate, if not it will be scheduled on the dispatch thread.
     *
     * @param objects
     */
    private void removeObjects(final List<LayerObject<?,?>> objects) {
        if (EventQueue.isDispatchThread()) {
            internalRemoveObjects(objects);
        } else {
            EventQueue.invokeLater(new Runnable() {

                @Override
                public void run() {
                    internalRemoveObjects(objects);
                }
            });
        }
    }

    private void internalRemoveObjects(List<LayerObject<?,?>> objects) {
        assert objects != null;
        for (LayerObject layerObject : objects) {
            List<Widget> widgets = this.scene.findWidgets(layerObject);
            if (widgets != null) {
                for (Widget w : widgets) {
                    mainLayer.removeChild(w);
                }
                this.scene.removeObject(layerObject);
            }

        }
        this.scene.validate();
    }

    /**
     * add objects
     * if called from the event dispatch thread it ill be
     * executed immediate, if not it will be scheduled on the dispatch thread.
     *
     * @param objects
     */
    private void addObjects(final List<LayerObject> objects) {
        if (SwingUtilities.isEventDispatchThread()) {
            internalAddObjects(objects);
        } else {
            SwingUtilities.invokeLater(new Runnable() {

                @Override
                public void run() {
                    internalAddObjects(objects);
                }
            });
        }
    }

    /**
     * Direct add
     * 
     * @param nodes
     */
    private void internalAddObjects(final List<LayerObject> objects) {
        for (LayerObject layerObject: objects) {
            addObject(layerObject);
        }
        scene.validate();
    }

    /**
     * Add an object to the panel.
     *
     * @param objectNode
     */
    private void addObject(final LayerObject object) {

        //TODO widget based on node?
        LayerObjectWidget widget = new LayerObjectWidget(this.scene, object, this.layer,controller);

        widget.getActions().addAction(scene.createObjectHoverAction());
        widget.getActions().addAction(scene.createSelectAction());




//        widget.getActions().addAction(
//        ActionFactory.createEditAction(new EditProvider() {
//
//            @Override
//            public void edit(Widget widget) {.
//                    ActionFactory.
//                throw new UnsupportedOperationException("Not supported yet.");
//            }
//        }));

//        //TODO: popup menu for widgets does not show and blocks event to map
//        PopupMenuProvider mp = new PopupMenuProvider() {
//
//            @Override
//            public JPopupMenu getPopupMenu(Widget widget, Point localLocation) {
//                JPopupMenu menu = new JPopupMenu();
//                menu.add(objectNode.getPreferredAction());
////                for (Action a:objectNode.getActions(true)){
////                    menu.add(a);
////                }
//                return menu;
//            }
//        };
//        widget.getActions().addAction(ActionFactory.createPopupMenuAction(mp));


        this.mainLayer.addChild(widget);
        this.scene.addObject(object, widget);

    }

    public boolean isInterActive() {
        return this.layer.isInterActive();
    }

    public void setSelectedObjects(Set<LayerObject> selectedObjects) {
        this.scene.setSelectedObjects(selectedObjects);
        this.scene.getView().repaint();
    }

    public void setSelectedObject(LayerObject selectedObject) {
        Set<LayerObject> selectedObjects = new HashSet<LayerObject>();
        selectedObjects.add(selectedObject);
        this.scene.setSelectedObjects(selectedObjects);
        this.scene.getView().repaint();
    }

 
}
