/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.panels;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Map;
import javax.swing.JComponent;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import nl.cloudfarming.client.geoviewer.LayerInfo;
import nl.cloudfarming.client.geoviewer.jxmap.FillLayout;
import org.jdesktop.swingx.JXMapViewer;
import org.openide.explorer.ExplorerManager;

/**
 *
 * @author Timon Veenstra
 */
public class RootMapPanel extends JPanel implements MouseListener, MouseMotionListener, MouseWheelListener {

    // swing container holding all layerPanels and the map panel
    private final JLayeredPane layerPanelContainer;
    // layers
    private static final Integer LAYER_MAP = new Integer(1);
    private static final Integer LAYER_OBJECT = new Integer(10);
    private static final Integer LAYER_OBJECT_INTERACTION = new Integer(20);
    private static final Integer LAYER_DRAWING = new Integer(30);
    // dragging variables
    private Point startPressedPoint;
    private int mouseMovementX;
    private int mouseMovementY;
    private int panOffsetX;
    private int panOffsetY;
    private boolean mouseDragging;
    private boolean drawOldStateImage;
    // image to be drawn while dragging so layers will drag in sync
    private BufferedImage layeredPaneDragImage;
    private JXMapPanel mapPanel;
    private DrawingLayerPanel drawingLayerPanel;
    //
    // map with layerpanels for components
    //
    private Map<Component, LayerInfo> layerInfoMap = new HashMap<Component, LayerInfo>();

    public RootMapPanel(ExplorerManager manager) {
        setLayout(new BorderLayout());
        //
        // the layered pane which will contain the layers of the map
        //
        layerPanelContainer = new JLayeredPane();
        layerPanelContainer.setLayout(new FillLayout());
        add(layerPanelContainer, BorderLayout.CENTER);
        //
        // the map layer
        //
        mapPanel = new JXMapPanel(manager);
        layerPanelContainer.add(mapPanel, LAYER_MAP);
        //
        // the drawing layer
        //
        drawingLayerPanel = new DrawingLayerPanel();
        layerPanelContainer.add(drawingLayerPanel, LAYER_DRAWING);
        //
        // mouse event listerner for drawing panel, top panel during dragging
        //
        drawingLayerPanel.addMouseListener(this);
        drawingLayerPanel.addMouseMotionListener(this);
        drawingLayerPanel.addMouseWheelListener(this);
        //
        // catch mouse events on the root map panel
        //
        addMouseListener(this);
        addMouseMotionListener(this);
        addMouseWheelListener(this);
    }

    @Override
    public void paint(Graphics grphcs) {
        super.paint(grphcs);

        if (this.drawOldStateImage) {
            // if panning, draw still image of old map panel state
            this.drawingLayerPanel.drawImage(this.layeredPaneDragImage, this.mouseMovementX + this.panOffsetX, this.mouseMovementY
                    + this.panOffsetY);
        }
    }

    private void dispatchToMap(MouseEvent e) {
        mapPanel.getMainMap().dispatchEvent(e);
    }

    @Override
    public void mouseClicked(MouseEvent me) {
        dispatchToMap(me);
    }

    @Override
    public void mousePressed(MouseEvent me) {
        this.startPressedPoint = me.getPoint();
        dispatchToMap(me);
    }

    @Override
    public void mouseReleased(MouseEvent e) {
        final Point point = e.getPoint();
        if (this.startPressedPoint != null) {
            this.mouseMovementX = (int) (point.getX() - this.startPressedPoint.getX());
            this.mouseMovementY = (int) (point.getY() - this.startPressedPoint.getY());
        }
        handleMouseReleasedPanMode();
        clearMouseMovement();

        this.drawOldStateImage = false;
        //
        // show object layers
        //
        for (Component c : this.layerPanelContainer.getComponentsInLayer(LAYER_OBJECT)) {
            if (this.layerInfoMap.containsKey(c) && this.layerInfoMap.get(c).isVisible()) {
                c.setVisible(true);
            }
        }
        for (Component c : this.layerPanelContainer.getComponentsInLayer(LAYER_OBJECT_INTERACTION)) {
            if (this.layerInfoMap.containsKey(c) && this.layerInfoMap.get(c).isVisible()) {
                c.setVisible(true);
            }
        }
        this.drawingLayerPanel.hideDrawingPanel();

        repaint();
        dispatchToMap(e);
    }

    @Override
    public void mouseEntered(MouseEvent me) {
        dispatchToMap(me);
    }

    @Override
    public void mouseExited(MouseEvent me) {
        dispatchToMap(me);
    }

    @Override
    public void mouseDragged(MouseEvent e) {
        final Point point = e.getPoint();
        if (this.startPressedPoint != null) {
            this.mouseMovementX = (int) (point.getX() - this.startPressedPoint.getX());
            this.mouseMovementY = (int) (point.getY() - this.startPressedPoint.getY());
        }
        handleMouseDraggingPanMode();

        repaint();
        dispatchToMap(e);
    }

    @Override
    public void mouseMoved(MouseEvent me) {
        dispatchToMap(me);
    }

    private void handleMouseDraggingPanMode() {
        if (!this.mouseDragging && !this.drawOldStateImage) {
            this.panOffsetX = 0;
            this.panOffsetY = 0;

            createPanImage();
        }
        this.mouseDragging = true;
    }

    private void createPanImage() {
        // create a still image of the current state of this map panel. This
        // image is following the mouse and thus
        // providing a visual clue that panning occurs
        // hide the underlying map panel when making the snapshot 
        mapPanel.setVisible(false);
        this.layeredPaneDragImage = createImage(layerPanelContainer, BufferedImage.TYPE_INT_ARGB);
        mapPanel.setVisible(true);

        this.drawingLayerPanel.paint(this.layeredPaneDragImage.getGraphics());
        //
        // hide object layers
        //
        for (Component c : this.layerPanelContainer.getComponentsInLayer(LAYER_OBJECT)) {
            c.setVisible(false);
        }
        for (Component c : this.layerPanelContainer.getComponentsInLayer(LAYER_OBJECT_INTERACTION)) {
            c.setVisible(false);
        }

        this.drawingLayerPanel.setVisible(true);

        // a separate flag indicating whether or not to draw the drag image is
        // needed.
        // mouseDragging is not sufficient because it is reset before the new
        // map is available.
        this.drawOldStateImage = true;
    }

    public static BufferedImage createImage(JComponent component, int imageType) {
        Dimension componentSize = component.getPreferredSize();
        component.setDoubleBuffered(false);
        component.setSize(componentSize);
//        component.addNotify();
        component.validate();

        BufferedImage img = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().createCompatibleImage(component.getSize().width,
                component.getSize().height, imageType);
        Graphics2D grap = img.createGraphics();
//        grap.setColor(Color.WHITE);
//        grap.fillRect(0, 0, img.getWidth(), img.getHeight());
        component.print(grap);
        grap.dispose();
        return img;
    }

    private BufferedImage ImageToBufferedImage(Image image, int width, int height) {
        BufferedImage dest = new BufferedImage(
                width, height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g2 = dest.createGraphics();
        g2.drawImage(image, 0, 0, null);
        g2.dispose();
        return dest;
    }

    private void handleMouseReleasedPanMode() {
        this.mouseDragging = false;

        if (this.mouseMovementX != 0 || this.mouseMovementY != 0) {
            this.panOffsetX += this.mouseMovementX;
            this.panOffsetY += this.mouseMovementY;
        }

        requestFocusInWindow();
    }

    private void clearMouseMovement() {
        this.mouseMovementX = 0;
        this.mouseMovementY = 0;
        this.startPressedPoint = null;
    }

    @Override
    public void mouseWheelMoved(MouseWheelEvent mwe) {
        dispatchToMap(mwe);
    }

    JXMapViewer getMapViewer() {
        return this.mapPanel.getMainMap();
    }

    void addLayerPanel(final LayerPanel layerPanel) {
        //
        // add interactive layers to the interaction layer
        // and add non interactive layer to the object layer
        //
        layerPanelContainer.add(layerPanel.getView(), (layerPanel.isInterActive()) ? LAYER_OBJECT_INTERACTION : LAYER_OBJECT);
        layerInfoMap.put(layerPanel.getView(), layerPanel);

        layerPanel.addPropertyChangeListener(new PropertyChangeListener() {

            @Override
            public void propertyChange(PropertyChangeEvent evt) {
                if (LayerPanel.PROPERTY_VISIBLE.equals(evt.getPropertyName())) {
                    layerPanel.getView().setVisible((Boolean) evt.getNewValue());
                }
            }
        });

    }

    void removeLayerPanel(LayerPanel layerPanel) {
        layerPanelContainer.remove(layerPanel.getView());
    }
}
