/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer.layers;

import java.beans.IntrospectionException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.swing.Action;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.LayerAction;
import nl.cloudfarming.client.geoviewer.LayerInfo;
import nl.cloudfarming.client.geoviewer.MapController;
import org.openide.nodes.BeanNode;
import org.openide.nodes.Children;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;

/**
 * A node which indicates a layer (image or items).
 * 
 * @author Timon Veenstra
 */
public class LayerNode extends BeanNode<Layer> implements Comparable {

    private LayerInfo layerInfo;

    public LayerNode(Layer layer, MapController mapController) throws IntrospectionException {
        // only display children of interactive layers
        super(layer, (layer.isInterActive()) ? new LayerObjectChildren(layer) : Children.LEAF, Lookups.fixed(layer, mapController));
        this.layerInfo = new LayerInfo();
    }

    public Layer getLayer() {
        return getBean();
    }

    /**
     * Returns the HTML display name based on the visibility of the layer.
     * 
     * @return The HTML display name of the node (layer)
     */
    @Override
    public String getHtmlDisplayName() {
        String displayName = getBean().getName();
        if (displayName == null) {
            displayName = getBean().getName();
        }

        if (layerInfo.isVisible()) {
            return "<font color='#000000'>" + displayName + "</font>";
        } else {
            return "<font color='#AAAAAA'>" + displayName + "</font>";
        }
    }

    /**
     * Returns the actions for a node (layer).
     * 
     * @param popup
     * @return
     */
    @Override
    public Action[] getActions(boolean popup) {
        // first add all global layer actions from the lookup
        List<LayerAction> globalActions = new ArrayList<LayerAction>( Lookup.getDefault().lookupAll(LayerAction.class));
        //
        // add layer specific actions
        //
        LayerAction[] specificActions = getBean().getActions();
        if (specificActions != null && specificActions.length > 0) {
            globalActions.addAll(Arrays.asList(specificActions));
        }
        return globalActions.toArray(new Action[]{});
    }

    /**
     * Required for the sortedArray interface for the parentNode.children
     * Dummy function. Only needed for the encapsulating Children.sortedArray class.
     * @param arg0
     * @return
     */
    @Override
    public int compareTo(Object arg0) {
        return 0; // no need to change.. it doesn't do anything for us
    }

    /**
     * Add support for copying this node
     */
    @Override
    public boolean canCopy() {
        return true;
    }

    /**
     * Add support for cutting (moving) this node
     */
    @Override
    public boolean canCut() {
        return true;
    }
}
